package il.ac.tau.cs.sw1.bioinformatics;

import java.io.IOException;
import org.apache.commons.math3.stat.inference.TestUtils;

/**
 * 
 * Gene Expression Analyzer
 * 
 * Command line arguments:
 *  args[0] - GeoDatasetName: Gene expression dataset name (expects a corresponding input file in SOFT format to exist in the local directory).
 *  args[1] - Label1: Label of the first sample subset
 *  args[2] - Label2: Label of the second sample subset
 *  args[3] - Alpha: T-test confidence level : only genes with pValue below this threshold will be printed to output file
 *  
 * Execution example: GeneExpressionAnalyzer GDS4085 "estrogen receptor-negative" "estrogen receptor-positive" 0.01
 *  
 * @author software1-2014
 *
 */
public class GeneExpressionAnalyzer {

	public static void main(String args[]) throws IOException {

		// Reads the dataset from a SOFT input file
		String inputSoftFileName = args[0] + ".soft";
		GeneExpressionDataset geneExpressionDataset = parseGeneExpressionFile (inputSoftFileName);
		System.out.printf ("Gene expression dataset loaded from file %s. %n",inputSoftFileName);
		System.out.printf("Dataset contains %d samples and %d gene probes.%n%n",geneExpressionDataset.samplesNumber, geneExpressionDataset.genesNumber);

		// Writes the dataset to a tabular format
		String tabularFileName = args[0] + "-Tabular.txt";
		writeDatasetToTabularFile(geneExpressionDataset,tabularFileName);
		System.out.printf ("Dataset saved to tabular file - %s.%n%n",tabularFileName);

		// Identifies differentially expressed genes between two sample groups and writes the results to a text file		
		String label1 = args[1]; 
		String label2 =  args[2]; 
		double alpha = Double.parseDouble(args[3]);
		String diffGenesFileName = args[0] + "-DiffGenes.txt";
		int numOfDiffGenes = writeTopDifferentiallyExpressedGenesToFile(diffGenesFileName,geneExpressionDataset, alpha, label1, label2);
		System.out.printf ("%d differentially expressed genes identified using alpha of %f when comparing the two sample groups [%s] and [%s].%n",numOfDiffGenes,  alpha, label1, label2);
		System.out.printf ("Results saved to file %s.%n",diffGenesFileName);
	}


	/**
	 * 
	 * parseGeneExpressionFile - parses the given SOFT file
	 * 
	 * 
	 * @param filename A gene expression file in SOFT format
	 * @return a GeneExpressionDataset object storing all data parsed from the input file
	 * @throws IOException
	 */
	public static GeneExpressionDataset parseGeneExpressionFile (String filename) throws IOException {

		GeneExpressionDataset dataset = new GeneExpressionDataset();

		// -----------------------------------------------------------------------------------------------------
		//TODO - Complete the code here -----------------------------------------------------------------
		// -----------------------------------------------------------------------------------------------------

		return dataset;

	}

	/**
	 * writeDatasetToTabularFile
	 * writes the dataset to a tabular text file
	 * 
	 * @param geneExpressionDataset
	 * @param outputFilename
	 * @throws IOException
	 */
	public static void writeDatasetToTabularFile(GeneExpressionDataset geneExpressionDataset, String outputFilename) throws IOException {

		// -----------------------------------------------------------------------------------------------------
		//TODO - Complete the code here -----------------------------------------------------------------
		// -----------------------------------------------------------------------------------------------------

	}


	/**
	 * 
	 * writeTopDifferentiallyExpressedGenesToFile
	 * 
	 * @param outputFilename
	 * @param geneExpressionDataset
	 * @param alpha
	 * @param label1
	 * @param label2
	 * @return numOfDiffGenes The number of differentially expressed genes detected, having p-value lower than alpha
	 * @throws IOException
	 */
	public static int writeTopDifferentiallyExpressedGenesToFile(String outputFilename,
			GeneExpressionDataset geneExpressionDataset, double alpha,
			String label1, String label2) throws IOException {

		double pValues[] = new double[geneExpressionDataset.genesNumber];

		// -----------------------------------------------------------------------------------------------------
		//TODO - Complete the code here -----------------------------------------------------------------
		// -----------------------------------------------------------------------------------------------------

	}

	/**
	 * 
	 * getDataEntriesForLabel
	 * 
	 * Returns the entries in the 'data' array for which the corresponding entries in the 'labels' array equals 'label'
	 * 
	 * @param data
	 * @param labels
	 * @param label
	 * @return
	 */
	public static double[] getDataEntriesForLabel(float[] data, String[] labels, String label) {

		// -----------------------------------------------------------------------------------------------------
		//TODO - Complete the code here -----------------------------------------------------------------
		// -----------------------------------------------------------------------------------------------------

	}

	/**
	 *   calcTtest - returns a pValue for the t-Test  
	 * 
	 *  Returns the p-value, associated with a two-sample, two-tailed t-test comparing the means of the input arrays
	 * 
	 * 		//http://commons.apache.org/proper/commons-math/apidocs/org/apache/commons/math3/stat/inference/TTest.html#tTest(double[], double[])
	 * 
	 * @param geneExpressionDataset
	 * @param geneIndex
	 * @param label1
	 * @param label2
	 * @return
	 */
	private static double calcTtest(GeneExpressionDataset geneExpressionDataset, int geneIndex, String label1, String label2) {
		double[] sample1 = getDataEntriesForLabel(geneExpressionDataset.dataMatrix[geneIndex],  geneExpressionDataset.labels, label1);
		double[] sample2 = getDataEntriesForLabel(geneExpressionDataset.dataMatrix[geneIndex],  geneExpressionDataset.labels, label2);
		return TestUtils.tTest(sample1, sample2);
	}

	/**
	 * 
	 *  GeneExpressionDataset
	 *  A class representing a gene expression dataset
	 * 
	 * @author software1-2014
	 *
	 */
	public static class GeneExpressionDataset {

		public int samplesNumber; //number of dataset samples
		public int genesNumber; // number of dataset gene probes

		public String[] sampleIds; //sample ids
		public String[] geneIds; //gene probe ids
		public String[] geneSymbols; //gene symbols
		public float[][] dataMatrix; //expression data matrix

		public String[] labels; //sample labels
	}

}
